@@ -133,6 +133,7 @@ public class ThrottlingHttpClientHandler extends NHttpHandlerBase
                 DirectByteBufferAllocator.INSTANCE, executor, params);
     }
 
+    @Override
     public void connected(final NHttpClientConnection conn, final Object attachment) {
         final HttpContext context = conn.getContext();
 
@@ -148,6 +149,7 @@ public class ThrottlingHttpClientHandler extends NHttpHandlerBase
         requestReady(conn);
     }
 
+    @Override
     public void closed(final NHttpClientConnection conn) {
         final HttpContext context = conn.getContext();
         final ClientConnState connState = (ClientConnState) context.getAttribute(CONN_STATE);
@@ -165,6 +167,7 @@ public class ThrottlingHttpClientHandler extends NHttpHandlerBase
         }
     }
 
+    @Override
     public void exception(final NHttpClientConnection conn, final HttpException ex) {
         closeConnection(conn, ex);
         if (this.eventListener != null) {
@@ -172,6 +175,7 @@ public class ThrottlingHttpClientHandler extends NHttpHandlerBase
         }
     }
 
+    @Override
     public void exception(final NHttpClientConnection conn, final IOException ex) {
         shutdownConnection(conn, ex);
         if (this.eventListener != null) {
@@ -180,6 +184,7 @@ public class ThrottlingHttpClientHandler extends NHttpHandlerBase
     }
 
 
+    @Override
     public void requestReady(final NHttpClientConnection conn) {
         final HttpContext context = conn.getContext();
 
@@ -240,6 +245,7 @@ public class ThrottlingHttpClientHandler extends NHttpHandlerBase
         }
     }
 
+    @Override
     public void outputReady(final NHttpClientConnection conn, final ContentEncoder encoder) {
         final HttpContext context = conn.getContext();
 
@@ -271,6 +277,7 @@ public class ThrottlingHttpClientHandler extends NHttpHandlerBase
         }
     }
 
+    @Override
     public void responseReceived(final NHttpClientConnection conn) {
         final HttpContext context = conn.getContext();
         final ClientConnState connState = (ClientConnState) context.getAttribute(CONN_STATE);
@@ -342,6 +349,7 @@ public class ThrottlingHttpClientHandler extends NHttpHandlerBase
         }
     }
 
+    @Override
     public void inputReady(final NHttpClientConnection conn, final ContentDecoder decoder) {
         final HttpContext context = conn.getContext();
 
@@ -374,6 +382,7 @@ public class ThrottlingHttpClientHandler extends NHttpHandlerBase
         }
     }
 
+    @Override
     public void timeout(final NHttpClientConnection conn) {
         final HttpContext context = conn.getContext();
         final ClientConnState connState = (ClientConnState) context.getAttribute(CONN_STATE);
@@ -436,6 +445,7 @@ public class ThrottlingHttpClientHandler extends NHttpHandlerBase
 
             this.executor.execute(new Runnable() {
 
+                @Override
                 public void run() {
                     try {
 
@@ -460,11 +470,11 @@ public class ThrottlingHttpClientHandler extends NHttpHandlerBase
                             connState.setWorkerRunning(true);
                         }
 
-                        final OutputStream outstream = new ContentOutputStream(
+                        final OutputStream outStream = new ContentOutputStream(
                                 connState.getOutbuffer());
-                        request.getEntity().writeTo(outstream);
-                        outstream.flush();
-                        outstream.close();
+                        request.getEntity().writeTo(outStream);
+                        outStream.flush();
+                        outStream.close();
 
                         synchronized (connState) {
                             connState.setWorkerRunning(false);
@@ -492,6 +502,7 @@ public class ThrottlingHttpClientHandler extends NHttpHandlerBase
 
         this.executor.execute(new Runnable() {
 
+            @Override
             public void run() {
                 try {
 
@@ -569,7 +580,7 @@ public class ThrottlingHttpClientHandler extends NHttpHandlerBase
         public static final int RESPONSE_BODY_DONE         = 64;
         public static final int RESPONSE_DONE              = 64;
 
-        private final SharedInputBuffer inbuffer;
+        private final SharedInputBuffer inBuffer;
         private final SharedOutputBuffer outbuffer;
 
         private volatile int inputState;
@@ -587,14 +598,14 @@ public class ThrottlingHttpClientHandler extends NHttpHandlerBase
                 final IOControl ioControl,
                 final ByteBufferAllocator allocator) {
             super();
-            this.inbuffer = new SharedInputBuffer(bufsize, ioControl, allocator);
+            this.inBuffer = new SharedInputBuffer(bufsize, ioControl, allocator);
             this.outbuffer = new SharedOutputBuffer(bufsize, ioControl, allocator);
             this.inputState = READY;
             this.outputState = READY;
         }
 
         public ContentInputBuffer getInbuffer() {
-            return this.inbuffer;
+            return this.inBuffer;
         }
 
         public ContentOutputBuffer getOutbuffer() {
@@ -650,21 +661,21 @@ public class ThrottlingHttpClientHandler extends NHttpHandlerBase
         }
 
         public void close() {
-            this.inbuffer.close();
+            this.inBuffer.close();
             this.outbuffer.close();
             this.inputState = SHUTDOWN;
             this.outputState = SHUTDOWN;
         }
 
         public void shutdown() {
-            this.inbuffer.shutdown();
+            this.inBuffer.shutdown();
             this.outbuffer.shutdown();
             this.inputState = SHUTDOWN;
             this.outputState = SHUTDOWN;
         }
 
         public void resetInput() {
-            this.inbuffer.reset();
+            this.inBuffer.reset();
             this.request = null;
             this.inputState = READY;
         }
