@@ -138,13 +138,13 @@ public class NHttpConnectionBase
         Args.notNull(session, "I/O session");
         Args.notNull(params, "HTTP params");
 
-        int buffersize = params.getIntParameter(CoreConnectionPNames.SOCKET_BUFFER_SIZE, -1);
-        if (buffersize <= 0) {
-            buffersize = 4096;
+        int bufferSize = params.getIntParameter(CoreConnectionPNames.SOCKET_BUFFER_SIZE, -1);
+        if (bufferSize <= 0) {
+            bufferSize = 4096;
         }
-        int linebuffersize = buffersize;
-        if (linebuffersize > 512) {
-            linebuffersize = 512;
+        int lineBufferSize = bufferSize;
+        if (lineBufferSize > 512) {
+            lineBufferSize = 512;
         }
 
         CharsetDecoder decoder = null;
@@ -162,9 +162,9 @@ public class NHttpConnectionBase
             decoder.onMalformedInput(malformedCharAction).onUnmappableCharacter(unmappableCharAction);
             encoder.onMalformedInput(malformedCharAction).onUnmappableCharacter(unmappableCharAction);
         }
-        this.inbuf = new SessionInputBufferImpl(buffersize, linebuffersize, decoder, allocator);
-        this.outbuf = new SessionOutputBufferImpl(buffersize, linebuffersize, encoder, allocator);
-        this.fragmentSizeHint = buffersize;
+        this.inbuf = new SessionInputBufferImpl(bufferSize, lineBufferSize, decoder, allocator);
+        this.outbuf = new SessionOutputBufferImpl(bufferSize, lineBufferSize, encoder, allocator);
+        this.fragmentSizeHint = bufferSize;
         this.constraints = MessageConstraints.DEFAULT;
 
         this.incomingContentStrategy = createIncomingContentStrategy();
@@ -184,14 +184,14 @@ public class NHttpConnectionBase
      * Creates new instance NHttpConnectionBase given the underlying I/O session.
      *
      * @param session the underlying I/O session.
-     * @param buffersize buffer size. Must be a positive number.
+     * @param bufferSize buffer size. Must be a positive number.
      * @param fragmentSizeHint fragment size hint.
      * @param allocator memory allocator.
      *   If {@code null} {@link org.apache.http.nio.util.HeapByteBufferAllocator#INSTANCE}
      *   will be used.
-     * @param chardecoder decoder to be used for decoding HTTP protocol elements.
+     * @param charDecoder decoder to be used for decoding HTTP protocol elements.
      *   If {@code null} simple type cast will be used for byte to char conversion.
-     * @param charencoder encoder to be used for encoding HTTP protocol elements.
+     * @param charEncoder encoder to be used for encoding HTTP protocol elements.
      *   If {@code null} simple type cast will be used for char to byte conversion.
      * @param constraints Message constraints. If {@code null}
      *   {@link MessageConstraints#DEFAULT} will be used.
@@ -204,23 +204,23 @@ public class NHttpConnectionBase
      */
     protected NHttpConnectionBase(
             final IOSession session,
-            final int buffersize,
+            final int bufferSize,
             final int fragmentSizeHint,
             final ByteBufferAllocator allocator,
-            final CharsetDecoder chardecoder,
-            final CharsetEncoder charencoder,
+            final CharsetDecoder charDecoder,
+            final CharsetEncoder charEncoder,
             final MessageConstraints constraints,
             final ContentLengthStrategy incomingContentStrategy,
             final ContentLengthStrategy outgoingContentStrategy) {
         Args.notNull(session, "I/O session");
-        Args.positive(buffersize, "Buffer size");
-        int linebuffersize = buffersize;
-        if (linebuffersize > 512) {
-            linebuffersize = 512;
+        Args.positive(bufferSize, "Buffer size");
+        int lineBufferSize = bufferSize;
+        if (lineBufferSize > 512) {
+            lineBufferSize = 512;
         }
-        this.inbuf = new SessionInputBufferImpl(buffersize, linebuffersize, chardecoder, allocator);
-        this.outbuf = new SessionOutputBufferImpl(buffersize, linebuffersize, charencoder, allocator);
-        this.fragmentSizeHint = fragmentSizeHint >= 0 ? fragmentSizeHint : buffersize;
+        this.inbuf = new SessionInputBufferImpl(bufferSize, lineBufferSize, charDecoder, allocator);
+        this.outbuf = new SessionOutputBufferImpl(bufferSize, lineBufferSize, charEncoder, allocator);
+        this.fragmentSizeHint = fragmentSizeHint >= 0 ? fragmentSizeHint : bufferSize;
 
         this.inTransportMetrics = new HttpTransportMetricsImpl();
         this.outTransportMetrics = new HttpTransportMetricsImpl();
@@ -239,14 +239,14 @@ public class NHttpConnectionBase
      * Creates new instance NHttpConnectionBase given the underlying I/O session.
      *
      * @param session the underlying I/O session.
-     * @param buffersize buffer size. Must be a positive number.
+     * @param bufferSize buffer size. Must be a positive number.
      * @param fragmentSizeHint fragment size hint.
      * @param allocator memory allocator.
      *   If {@code null} {@link org.apache.http.nio.util.HeapByteBufferAllocator#INSTANCE}
      *   will be used.
-     * @param chardecoder decoder to be used for decoding HTTP protocol elements.
+     * @param charDecoder decoder to be used for decoding HTTP protocol elements.
      *   If {@code null} simple type cast will be used for byte to char conversion.
-     * @param charencoder encoder to be used for encoding HTTP protocol elements.
+     * @param charEncoder encoder to be used for encoding HTTP protocol elements.
      *   If {@code null} simple type cast will be used for char to byte conversion.
      * @param incomingContentStrategy incoming content length strategy. If {@code null}
      *   {@link LaxContentLengthStrategy#INSTANCE} will be used.
@@ -257,14 +257,14 @@ public class NHttpConnectionBase
      */
     protected NHttpConnectionBase(
             final IOSession session,
-            final int buffersize,
+            final int bufferSize,
             final int fragmentSizeHint,
             final ByteBufferAllocator allocator,
-            final CharsetDecoder chardecoder,
-            final CharsetEncoder charencoder,
+            final CharsetDecoder charDecoder,
+            final CharsetEncoder charEncoder,
             final ContentLengthStrategy incomingContentStrategy,
             final ContentLengthStrategy outgoingContentStrategy) {
-        this(session, buffersize, fragmentSizeHint, allocator, chardecoder, charencoder,
+        this(session, bufferSize, fragmentSizeHint, allocator, charDecoder, charEncoder,
                 null, incomingContentStrategy, outgoingContentStrategy);
     }
 
@@ -365,7 +365,11 @@ public class NHttpConnectionBase
 
     @Override
     public void suspendOutput() {
-        this.session.clearEvent(EventMask.WRITE);
+        synchronized (this.session) {
+            if (!this.outbuf.hasData()) {
+                this.session.clearEvent(EventMask.WRITE);
+            }
+        }
     }
 
     /**
@@ -495,7 +499,7 @@ public class NHttpConnectionBase
      */
     protected void assertNotClosed() throws ConnectionClosedException {
         if (this.status != ACTIVE) {
-            throw new ConnectionClosedException("Connection is closed");
+            throw new ConnectionClosedException();
         }
     }
 
@@ -526,41 +530,25 @@ public class NHttpConnectionBase
     @Override
     public InetAddress getLocalAddress() {
         final SocketAddress address = this.session.getLocalAddress();
-        if (address instanceof InetSocketAddress) {
-            return ((InetSocketAddress) address).getAddress();
-        } else {
-            return null;
-        }
+        return address instanceof InetSocketAddress ? ((InetSocketAddress) address).getAddress() : null;
     }
 
     @Override
     public int getLocalPort() {
         final SocketAddress address = this.session.getLocalAddress();
-        if (address instanceof InetSocketAddress) {
-            return ((InetSocketAddress) address).getPort();
-        } else {
-            return -1;
-        }
+        return address instanceof InetSocketAddress ? ((InetSocketAddress) address).getPort() : -1;
     }
 
     @Override
     public InetAddress getRemoteAddress() {
         final SocketAddress address = this.session.getRemoteAddress();
-        if (address instanceof InetSocketAddress) {
-            return ((InetSocketAddress) address).getAddress();
-        } else {
-            return null;
-        }
+        return address instanceof InetSocketAddress ? ((InetSocketAddress) address).getAddress() : null;
     }
 
     @Override
     public int getRemotePort() {
         final SocketAddress address = this.session.getRemoteAddress();
-        if (address instanceof InetSocketAddress) {
-            return ((InetSocketAddress) address).getPort();
-        } else {
-            return -1;
-        }
+        return address instanceof InetSocketAddress ? ((InetSocketAddress) address).getPort() : -1;
     }
 
     @Override
@@ -594,18 +582,13 @@ public class NHttpConnectionBase
             buffer.append("<->");
             NetUtils.formatAddress(buffer, remoteAddress);
             return buffer.toString();
-        } else {
-            return "[Not bound]";
         }
+        return "[Not bound]";
     }
 
     @Override
     public Socket getSocket() {
-        if (this.session instanceof SocketAccessor) {
-            return ((SocketAccessor) this.session).getSocket();
-        } else {
-            return null;
-        }
+        return this.session instanceof SocketAccessor ? ((SocketAccessor) this.session).getSocket() : null;
     }
 
 }
