@@ -35,6 +35,7 @@ import java.nio.channels.ByteChannel;
 import java.nio.channels.CancelledKeyException;
 import java.nio.channels.ClosedChannelException;
 import java.nio.channels.SelectionKey;
+import java.util.concurrent.atomic.AtomicInteger;
 
 import javax.net.ssl.SSLContext;
 import javax.net.ssl.SSLEngine;
@@ -89,6 +90,7 @@ public class SSLIOSession implements IOSession, SessionBufferStatus, SocketAcces
     private final SSLBuffer inPlain;
     private final InternalByteChannel channel;
     private final SSLSetupHandler handler;
+    private final AtomicInteger outboundClosedCount;
 
     private int appEventMask;
     private SessionBufferStatus appBufferStatus;
@@ -163,6 +165,7 @@ public class SSLIOSession implements IOSession, SessionBufferStatus, SocketAcces
         // Allocate buffers for application (unencrypted) data
         final int appBuffersize = this.sslEngine.getSession().getApplicationBufferSize();
         this.inPlain = bufferManagementStrategy.constructBuffer(appBuffersize);
+        this.outboundClosedCount = new AtomicInteger(0);
     }
 
     /**
@@ -229,7 +232,11 @@ public class SSLIOSession implements IOSession, SessionBufferStatus, SocketAcces
             break;
         }
         if (this.handler != null) {
-            this.handler.initalize(this.sslEngine);
+            try {
+                this.handler.initalize(this.sslEngine);
+            } catch (final RuntimeException ex) {
+                throw convert(ex);
+            }
         }
         this.initialized = true;
         this.sslEngine.beginHandshake();
@@ -290,7 +297,15 @@ public class SSLIOSession implements IOSession, SessionBufferStatus, SocketAcces
 
         SSLEngineResult result = null;
         while (handshaking) {
-            switch (this.sslEngine.getHandshakeStatus()) {
+            HandshakeStatus handshakeStatus = this.sslEngine.getHandshakeStatus();
+
+            // Work-around for what appears to be a bug in Conscrypt SSLEngine that does not
+            // transition into the handshaking state upon #closeOutbound() call but still
+            // has some handshake data stuck in its internal buffer.
+            if (handshakeStatus == HandshakeStatus.NOT_HANDSHAKING && outboundClosedCount.get() > 0) {
+                handshakeStatus = HandshakeStatus.NEED_WRAP;
+            }
+            switch (handshakeStatus) {
             case NEED_WRAP:
                // Generate outgoing handshake data
 
@@ -313,9 +328,11 @@ public class SSLIOSession implements IOSession, SessionBufferStatus, SocketAcces
 
                 // Perform operations
                 inEncryptedBuf.flip();
-                result = doUnwrap(inEncryptedBuf, inPlainBuf);
-                inEncryptedBuf.compact();
-
+                try {
+                    result = doUnwrap(inEncryptedBuf, inPlainBuf);
+                } finally {
+                    inEncryptedBuf.compact();
+                }
 
                 try {
                     if (!inEncryptedBuf.hasRemaining() && result.getHandshakeStatus() == HandshakeStatus.NEED_UNWRAP) {
@@ -364,6 +381,7 @@ public class SSLIOSession implements IOSession, SessionBufferStatus, SocketAcces
         }
         if (this.status == CLOSING && !this.outEncrypted.hasData()) {
             this.sslEngine.closeOutbound();
+            this.outboundClosedCount.incrementAndGet();
         }
         if (this.status == CLOSING && this.sslEngine.isOutboundDone()
                 && (this.endOfStream || this.sslEngine.isInboundDone())
@@ -399,7 +417,9 @@ public class SSLIOSession implements IOSession, SessionBufferStatus, SocketAcces
             break;
         }
 
-        if (this.endOfStream && (this.appBufferStatus == null || !this.appBufferStatus.hasBufferedInput())) {
+        if (this.endOfStream &&
+                !this.inPlain.hasData() &&
+                (this.appBufferStatus == null || !this.appBufferStatus.hasBufferedInput())) {
             newMask = newMask & ~EventMask.READ;
         }
 
@@ -426,10 +446,14 @@ public class SSLIOSession implements IOSession, SessionBufferStatus, SocketAcces
         // Acquire buffer
         final ByteBuffer outEncryptedBuf = this.outEncrypted.acquire();
 
+        final int bytesWritten;
         // Perform operation
         outEncryptedBuf.flip();
-        final int bytesWritten = this.session.channel().write(outEncryptedBuf);
-        outEncryptedBuf.compact();
+        try {
+            bytesWritten = this.session.channel().write(outEncryptedBuf);
+        } finally {
+            outEncryptedBuf.compact();
+        }
 
         // Release if empty
         if (outEncryptedBuf.position() == 0) {
@@ -466,10 +490,14 @@ public class SSLIOSession implements IOSession, SessionBufferStatus, SocketAcces
             final ByteBuffer inEncryptedBuf = this.inEncrypted.acquire();
             final ByteBuffer inPlainBuf = this.inPlain.acquire();
 
+            final SSLEngineResult result;
             // Perform operations
             inEncryptedBuf.flip();
-            final SSLEngineResult result = doUnwrap(inEncryptedBuf, inPlainBuf);
-            inEncryptedBuf.compact();
+            try {
+                result = doUnwrap(inEncryptedBuf, inPlainBuf);
+            } finally {
+                inEncryptedBuf.compact();
+            }
 
             try {
                 if (!inEncryptedBuf.hasRemaining() && result.getHandshakeStatus() == HandshakeStatus.NEED_UNWRAP) {
@@ -610,7 +638,6 @@ public class SSLIOSession implements IOSession, SessionBufferStatus, SocketAcces
         if (this.session.getSocketTimeout() == 0) {
             this.session.setSocketTimeout(1000);
         }
-        this.sslEngine.closeOutbound();
         try {
             updateEventMask();
         } catch (final CancelledKeyException ex) {
